<?php
/// Copyright (c) 2004-2007, Tatter & Company / Tatter & Friends.
/// All rights reserved. Licensed under the GPL.
/// See the GNU General Public License for more details. (/doc/LICENSE, /doc/COPYRIGHT)
class RefererLog {
	function RefererLog() {
		$this->reset();
	}

	function reset() {
		$this->error =
		$this->host =
		$this->url =
		$this->referred =
			null;
	}
	
	function open($filter = '', $fields = '*', $sort = 'referred DESC') {
		global $database, $cubrid_con, $owner;
		if (is_numeric($filter))
			$filter = 'AND id = ' . $filter;
		else if (!empty($filter))
			$filter = 'AND ' . $filter;
		if (!empty($sort))
			$sort = 'ORDER BY ' . $sort;
		$this->close();
		$this->_result = cubrid_execute($cubrid_con,"SELECT $fields FROM {$database['prefix']}RefererLogs WHERE owner = $owner $filter $sort");
		if ($this->_result) {
			if ($this->_count = cubrid_num_rows($this->_result))
				return $this->shift();
			else
				cubrid_commit($cubrid_con);
		}
		unset($this->_result);
		return false;
	}
	
	function close() {
		if (isset($this->_result)) {
			cubrid_close_request($this->_result);
			unset($this->_result);
		}
		$this->_count = 0;
		$this->reset();
	}
	
	function shift() {
	  global $cubrid_con;
	  
		$this->reset();
		if ($this->_result && ($row = cubrid_fetch($this->_result))) {
			foreach ($row as $name => $value) {
				if ($name == 'owner')
					continue;
				$this->$name = $value;
			}
			return true;
		}
		return false;
	}
	
	function add($compile = true) {
		if (!isset($this->url))
			return $this->_error('url');
		$this->host = null;
		if (!$query = $this->_buildQuery())
			return false;
		if (!$query->hasAttribute('referred'))
			$query->setAttribute('referred', "SYS_TIMESTAMP - to_timestamp('09:00:00 AM 01/01/1970')");

		if (!$query->insert())
			return $this->_error('insert');
		
		if ($compile) {
			RefererStatistics::compile($this->host);
		}
		return true;
	}
	
	function getCount() {
		return (isset($this->_count) ? $this->_count : 0);
	}

	function _buildQuery() {
		global $database, $owner;
		$query = new TableQuery($database['prefix'] . 'RefererLogs');
		$query->setQualifier('owner', $owner);
		if (isset($this->host)) {
			$this->host = cubrid_lessen(trim($url['host']), 64);
			if (empty($this->host))
				return $this->_error('host');
			$query->setAttribute('host', $this->host, true);
		}
		if (isset($this->url)) {
			$this->url = cubrid_lessen(trim($this->url), 255);
			if (empty($this->url))
				return $this->_error('url');
			$url = parse_url($this->url);
			if (empty($url['host']))
				return $this->_error('url');
			$this->host = cubrid_lessen(trim($url['host']), 64);
			$query->setAttribute('host', $this->host, true);
			if (empty($url['scheme']))
				$this->url = 'http://' . $this->url;
			$query->setAttribute('url', $this->url, true);
		}
		if (isset($this->referred)) {
			if (!Validator::number($this->referred, 1))
				return $this->_error('referred');
			$query->setAttribute('referred', $this->referred);
		}
		return $query;
	}

	function _error($error) {
		$this->error = $error;
		return false;
	}
}
?>