<?php
/// Copyright (c) 2004-2007, Tatter & Company / Tatter & Friends.
/// All rights reserved. Licensed under the GPL.
/// See the GNU General Public License for more details. (/doc/LICENSE, /doc/COPYRIGHT)
class SkinSetting {
	function SkinSetting() {
		$this->reset();
	}

	function reset() {
		$this->error =
		$this->skin =
		$this->entriesOnRecent =
		$this->commentsOnRecent =
		$this->trackbacksOnRecent =
		$this->commentsOnGuestbook =
		$this->tagsOnTagbox =
		$this->alignOnTagbox =
		$this->expandComment =
		$this->expandTrackback =
		$this->recentNoticeLength =
		$this->recentEntryLength =
		$this->recentTrackbackLength =
		$this->linkLength =
		$this->showListOnCategory =
		$this->showListOnArchive =
		$this->tree =
		$this->colorOnTree =
		$this->bgColorOnTree =
		$this->activeColorOnTree =
		$this->activeBgColorOnTree =
		$this->labelLengthOnTree =
		$this->showValueOnTree =
			null;
	}
	
	function load($fields = '*') {
		global $database, $cubrid_con, $owner;
		$this->reset();
		if ($result = cubrid_execute($cubrid_con,"SELECT $fields FROM {$database['prefix']}SkinSettings WHERE owner = $owner")) {
			if ($row = cubrid_fetch($result)) {
				foreach ($row as $name => $value) {
					if ($name == 'owner')
						continue;
					switch ($name) {
						case 'tagboxalign':
							$name = 'alignOnTagbox';
							break;
					}
					$this->$name = $value;
				}
				cubrid_commit($cubrid_con);
				return true;
			}
			cubrid_commit($cubrid_con);
		}
		return false;
	}
	
	function save() {
		global $database, $owner;
		
		$query = new TableQuery($database['prefix'] . 'SkinSettings');
		$query->setQualifier('owner', $owner);
		if (isset($this->skin)) {
			if (strncmp($this->skin, 'customize/', 10) == 0) {
				if (strcmp($this->skin, "customize/$owner") != 0)
					return $this->_error('skin');
			} else {
				if (!Validator::filename($this->skin))
					return $this->_error('skin');
			}
			if (!Validator::path($this->skin) || !file_exists(ROOT . '/skin/' . $this->skin))
				return $this->_error('skin');
			$query->setAttribute('skin', $this->skin, false);
		}
		if (isset($this->entriesOnRecent)) {
			if (!Validator::number($this->entriesOnRecent, 1))
				return $this->_error('entriesonrecent');
			$query->setAttribute('entriesonrecent', $this->entriesOnRecent);
		}
		if (isset($this->commentsOnRecent)) {
			if (!Validator::number($this->commentsOnRecent, 1))
				return $this->_error('commentsonrecent');
			$query->setAttribute('commentsonrecent', $this->commentsOnRecent);
		}
		if (isset($this->trackbacksOnRecent)) {
			if (!Validator::number($this->trackbacksOnRecent, 1))
				return $this->_error('trackbacksonrecent');
			$query->setAttribute('trackbacksonrecent', $this->trackbacksOnRecent);
		}
		if (isset($this->commentsOnGuestbook)) {
			if (!Validator::number($this->commentsOnGuestbook, 1))
				return $this->_error('commentsonguestbook');
			$query->setAttribute('commentsonguestbook', $this->commentsOnGuestbook);
		}
		if (isset($this->tagsOnTagbox)) {
			if (!Validator::number($this->tagsOnTagbox, 1))
				return $this->_error('tagsontagbox');
			$query->setAttribute('tagsontagbox', $this->tagsOnTagbox);
		}
		if (isset($this->alignOnTagbox)) {
			if (!Validator::number($this->alignOnTagbox, 1, 3))
				return $this->_error('alignOnTagbox');
			$query->setAttribute('tagboxalign', $this->alignOnTagbox);
		}
		if (isset($this->expandComment))
			$query->setAttribute('expandcomment', Validator::getBit($this->expandComment));
		if (isset($this->expandTrackback))
			$query->setAttribute('expandtrackback', Validator::getBit($this->expandTrackback));
		if (isset($this->recentNoticeLength)) {
			if (!Validator::number($this->recentNoticeLength, 0))
				return $this->_error('recentnoticelength');
			$query->setAttribute('recentnoticelength', $this->recentNoticeLength);
		}
		if (isset($this->recentTrackbackLength)) {
			if (!Validator::number($this->recentTrackbackLength, 0))
				return $this->_error('recenttrackbacklength');
			$query->setAttribute('recenttrackbacklength', $this->recentTrackbackLength);
		}
		if (isset($this->linkLength)) {
			if (!Validator::number($this->linkLength, 0))
				return $this->_error('linklength');
			$query->setAttribute('linklength', $this->linkLength);
		}
		if (isset($this->showListOnCategory))
			$query->setAttribute('showlistoncategory', Validator::getBit($this->showListOnCategory));
		if (isset($this->showListOnArchive))
			$query->setAttribute('showlistonarchive', Validator::getBit($this->showListOnArchive));
		if (isset($this->tree)) {
			if (!Validator::directory($this->tree) || !file_exists(ROOT . '/image/tree/' . $this->tree))
				return $this->_error('tree');
			$query->setAttribute('tree', $this->tree, false);
		}
		if (isset($this->colorOnTree))
			$query->setAttribute('colorontree', $this->colorOnTree, true);
		if (isset($this->bgColorOnTree))
			$query->setAttribute('bgcolorontree', $this->bgColorOnTree, true);
		if (isset($this->activeColorOnTree))
			$query->setAttribute('activecolorontree', $this->activeColorOnTree, true);
		if (isset($this->activeBgColorOnTree))
			$query->setAttribute('activebgcolorontree', $this->activeBgColorOnTree, true);
		if (isset($this->labelLengthOnTree)) {
			if (!Validator::number($this->labelLengthOnTree, 0))
				return $this->_error('labellengthontree');
			$query->setAttribute('labellengthontree', $this->labelLengthOnTree);
		}
		if (isset($this->showValueOnTree))
			$query->setAttribute('showvalueontree', Validator::getBit($this->showValueOnTree));
			
		if ($query->update())
			return true;
		return $query->insert();
	}

	function _error($error) {
		$this->error = $error;
		return false;
	}
}
?>