<?php
/// Copyright (c) 2004-2007, Tatter & Company / Tatter & Friends.
/// All rights reserved. Licensed under the GPL.
/// See the GNU General Public License for more details. (/doc/LICENSE, /doc/COPYRIGHT)
class Trackback {
	function Trackback() {
		$this->reset();
	}

	function reset() {
		$this->error =
		$this->id =
		$this->entry =
		$this->url =
		$this->site =
		$this->title =
		$this->excerpt =
		$this->ip =
		$this->received =
		$this->isFiltered =
			null;
	}
	
	function open($filter = '', $fields = '*', $sort = 'written') {
		global $database, $cubrid_con, $owner;
		if (is_numeric($filter))
			$filter = 'AND id = ' . $filter;
		else if (!empty($filter))
			$filter = 'AND ' . $filter;
		if (!empty($sort))
			$sort = 'ORDER BY ' . $sort;
		$this->close();
		$this->_result = cubrid_execute($cubrid_con,"SELECT $fields FROM {$database['prefix']}Trackbacks WHERE owner = $owner $filter $sort");
		if ($this->_result) {
			if ($this->_count = cubrid_num_rows($this->_result))
				return $this->shift();
			else
				cubrid_commit($cubrid_con);
		}
		unset($this->_result);
		return false;
	}
	
	function close() {
		if (isset($this->_result)) {
			cubrid_close_request($this->_result);
			unset($this->_result);
		}
		$this->_count = 0;
		$this->reset();
	}
	
	function shift() {
	  global $cubrid_con;
	  
		$this->reset();
		if ($this->_result && ($row = cubrid_fetch($this->_result))) {
			foreach ($row as $name => $value) {
				if ($name == 'owner')
					continue;
				switch ($name) {
					case 'subject':
						$name = 'title';
						break;
					case 'written':
						$name = 'received';
						break;
				}
				$this->$name = $value;
			}
			return true;
		}
		return false;
	}
	
	function add() {
		global $database, $cubrid_con, $owner;
		$this->id = null;
		if (!isset($this->entry))
			return $this->_error('entry');
		if (!isset($this->url))
			return $this->_error('url');
		if (!isset($this->ip))
			$this->ip = $_SERVER['REMOTE_ADDR'];
		if (!$query = $this->_buildQuery())
			return false;
		if (!$query->hasAttribute('written'))
			$query->setAttribute('written', "SYS_TIMESTAMP - to_timestamp('09:00:00 AM 01/01/1970')");
		if (!isset($this->isFiltered))
			$this->isFiltered = 0;
		
		if (!$query->insert())
			return $this->_error('insert');
		$this->id = $query->id;

		if ($this->isFiltered == 0) {
			cubrid_execute($cubrid_con,"UPDATE {$database['prefix']}Entries SET trackbacks = trackbacks + 1 WHERE owner = $owner AND id = {$this->entry}");
			cubrid_commit($cubrid_con);
		}
		return true;
	}
	
	function getCount() {
		return (isset($this->_count) ? $this->_count : 0);
	}
	
	function _buildQuery() {
		global $database, $owner;
		$query = new TableQuery($database['prefix'] . 'Trackbacks');
		$query->setQualifier('owner', $owner);
		if (isset($this->id)) {
			if (!Validator::number($this->id, 1))
				return $this->_error('id');
			$query->setQualifier('id', $this->id);
		}
		if (isset($this->entry)) {
			if (!Validator::number($this->entry, 1))
				return $this->_error('entry');
			$query->setQualifier('entry', $this->entry);
		}
		if (isset($this->url)) {
			$this->url = cubrid_lessen(trim($this->url), 255);
			if (empty($this->url))
				return $this->_error('url');
			$query->setQualifier('url', $this->url, true);
		}
		if (isset($this->site)) {
			$this->site = cubrid_lessen(trim($this->site), 255);
			$query->setAttribute('site', $this->site, true);
		}
		if (isset($this->title)) {
			$this->title = cubrid_lessen(trim($this->title), 255);
			$query->setAttribute('subject', $this->title, true);
		}
		if (isset($this->excerpt)) {
			$this->excerpt = cubrid_lessen(trim($this->excerpt), 255);
			$query->setAttribute('excerpt', $this->excerpt, true);
		}
		if (isset($this->ip)) {
			if (!Validator::ip($this->ip))
				return $this->_error('ip');
			$query->setAttribute('ip', $this->ip, true);
		}
		if (isset($this->received)) {
			if (!Validator::timestamp($this->received))
				return $this->_error('received');
			$query->setAttribute('written', $this->received);
		}
		if (isset($this->isFiltered)) {
			if ($this->isFiltered) {
				$query->setAttribute('isfiltered', "SYS_TIMESTAMP - to_timestamp('09:00:00 AM 01/01/1970')");
			} else {
				$query->setAttribute('isfiltered', Validator::getBit($this->isFiltered));
			}
			
		}
		return $query;
	}

	function _error($error) {
		$this->error = $error;
		return false;
	}
}
?>
